import pygame

from Core.EventManager import Event
from Core.Parts.SpritePart import SpritePart
from Core.Parts.TextPart import TextPart


class EntityManager:
    def __init__(self, event_manager, screen, background):
        self.entities = []
        self.entities_to_add = []
        self.entities_to_remove = []

        self.all_sprites = []
        self.layered_dirty = None

        self.event_manager = event_manager

        self.event_manager.register(Event.CREATE_ENTITY, self.add)
        self.event_manager.register(Event.REMOVE_ENTITY, self.remove)

        self.screen = screen
        self.background = background

    def add(self, entity):
        entity.event_manager = self.event_manager
        entity.initialize()
        self.entities_to_add.append(entity)

    def add_all(self, entities):
        for e in entities:
            self.add(e)

    def remove(self, entity):
        entity.cleanup()
        self.entities_to_remove.append(entity)

    def clear(self):
        for e in self.entities:
            self.remove(e)
        self.update(0)

    def update(self, delta):
        for e in self.entities:
            e.update(delta)

        while self.entities_to_add:
            to_add = self.entities_to_add.pop()
            self.entities.append(to_add)
            if to_add.has(SpritePart):
                self.all_sprites.append(to_add.get(SpritePart).dirty_sprite)
                self.create_layered_dirty()
            if to_add.has(TextPart):    # TODO: make sprite component abstract
                self.all_sprites.append(to_add.get(TextPart).dirty_sprite)
                self.create_layered_dirty()

        while self.entities_to_remove:
            to_remove = self.entities_to_remove.pop()
            if to_remove in self.entities:
                self.entities.remove(to_remove)
            if to_remove.has(SpritePart):
                sprite = to_remove.get(SpritePart).dirty_sprite
                if sprite in self.all_sprites:
                    self.all_sprites.remove(sprite)
                    self.create_layered_dirty()
            if to_remove.has(TextPart):    # TODO: make sprite component abstract
                sprite = to_remove.get(TextPart).dirty_sprite
                if sprite in self.all_sprites:
                    self.all_sprites.remove(sprite)
                    self.create_layered_dirty()

    def create_layered_dirty(self):
        self.layered_dirty = pygame.sprite.LayeredDirty(self.all_sprites)
        self.layered_dirty.clear(self.screen, self.background)
