from events import Events
from enum import Enum


class Event(Enum):
    CREATE_ENTITY = 1
    REMOVE_ENTITY = 2
    ENTITY_DEAD = 3
    ENTITY_TOOK_HIT = 4
    PAUSE = 5
    UNPAUSE = 6
    RESTART = 7


class EventManager:

    def __init__(self):
        self.events = {}

    def invoke(self, event, *args):
        if event in self.events:
            self.events[event].on_change(*args)

    def register(self, event, callback):
        if event not in self.events:
            self.events[event] = Events()

        self.events[event].on_change += callback

    def unregister(self, event, callback):
        if event in self.events:
            if callback in self.events[event]:
                self.events[event] -= callback

    def clear(self):
        self.events = {}
