from enum import Enum

import math

from Core.MathUtils import distance
from Core.Parts.Part import Part


class ColliderType(Enum):
    Box = 1
    Circle = 2


class BaseColliderPart(Part):
    def __init__(self, collider_type):
        super().__init__()
        self.collider_type = collider_type

    def collide_box_box(self, box1, box2):
        return box1.rect.colliderect(box2.rect)

    def collide_circ_circ(self, circ1, circ2):
        dist = distance(circ1.position, circ2.position)
        return dist < circ1.radius + circ2.radius

    def collide_box_circ(self, box, circ):
        """ Detect collision between a rectangle and circle. """

        c_center = circ.position
        r = circ.radius

        b_center = box.rect.center
        b_w = box.rect.width
        b_h = box.rect.height

        circle_distance_x = abs(c_center[0] - b_center[0])
        circle_distance_y = abs(c_center[1] - b_center[1])
        if circle_distance_x > b_w / 2.0 + r or circle_distance_y > b_h / 2.0 + r:
            return False
        if circle_distance_x <= b_w / 2.0 or circle_distance_y <= b_h / 2.0:
            return True
        corner_x = circle_distance_x - b_w / 2.0
        corner_y = circle_distance_y - b_h / 2.0
        corner_distance_sq = corner_x ** 2.0 + corner_y ** 2.0
        return corner_distance_sq <= r ** 2.0

    def collides_with_collider(self, other_collider):
        if self.collider_type == ColliderType.Box:
            if other_collider.collider_type == ColliderType.Circle:
                return self.collide_box_circ(self, other_collider)
            elif other_collider.collider_type == ColliderType.Box:
                return self.collide_box_box(self, other_collider)
        if self.collider_type == ColliderType.Circle:
            if other_collider.collider_type == ColliderType.Circle:
                return self.collide_circ_circ(self, other_collider)
            elif other_collider.collider_type == ColliderType.Box:
                return self.collide_box_box(other_collider, self)
