from random import randint, random

from Core.MathUtils import normalize, sub, random_sign
from Core.Parts.MissileLauncherPart import MissileLauncherPart
from Core.Parts.OnCircleMovementPart import OnCircleMovementPart
from Core.Parts.Part import Part
from Core.Parts.TransformPart import TransformPart
from Core.Parts.VesselPart import VesselPart


class EnemyAIPart(Part):
    def __init__(self):
        super().__init__()

        self.movement_part = None
        self.missile_launcher = None
        self.vessel = None

        self.time_from_last_mov_dir_change = 0
        self.next_mov_dir_change = 0
        self.move_dir_change_interval_range = (1, 15)

        self.shot_delay = 1.0
        self.curr_shot_delay = 0

    def initialize(self):
        self.movement_part = self.entity.get(OnCircleMovementPart)
        if self.entity.has(MissileLauncherPart):
            self.missile_launcher = self.entity.get(MissileLauncherPart)
        self.transform = self.entity.get(TransformPart)
        self.vessel = self.entity.get(VesselPart)

        self.time_from_last_mov_dir_change = 0
        self.next_mov_dir_change = self._next_mov_dir_change_time()
        self.curr_shot_delay = self._random_shot_delay

    def update(self, delta):

        if not self.vessel.mov_dir:
            self.vessel.mov_dir = random_sign()

        self.time_from_last_mov_dir_change += delta

        if self.time_from_last_mov_dir_change >= self.next_mov_dir_change and self._is_well_hidden:
            self.vessel.mov_dir *= -1
            self.time_from_last_mov_dir_change = 0
            self.next_mov_dir_change = self._next_mov_dir_change_time()

        if self._is_on_open_side:
            self.curr_shot_delay -= delta
            if self.missile_launcher and self.curr_shot_delay <= 0:
                self.shoot()

    @property
    def vec_up(self):
        return normalize(sub(self.transform.position, self.movement_part.planet_transform.position))

    def _next_mov_dir_change_time(self):
        r = self.move_dir_change_interval_range
        return randint(r[0], r[1])

    @property
    def _is_on_open_side(self):
        return self.transform.position[0] < self.movement_part.planet_transform.position[0]

    @property
    def _is_well_hidden(self):
        """When enemy is on the hidden side of the planet and align horizontally with planet origin"""
        y_offset = self.movement_part.planet_transform.position[1] - self.transform.position[1]
        y_aligned = abs(y_offset) <= self.movement_part.radius / 8
        return not self._is_on_open_side and y_aligned

    def shoot(self):
        self.missile_launcher.launch_missile(self.vec_up)
        self.curr_shot_delay = self._random_shot_delay

    @property
    def _random_shot_delay(self):
        return random() * self.shot_delay
