from Core.Entity import Entity
from Core.MathUtils import normalize, sub, add, scale
from Core.Parts.AudioSourcePart import AudioSourcePart
from Core.Parts.BoxColliderPart import BoxColliderPart
from Core.Parts.MissileControlPart import MissileControlPart
from Core.Parts.MissileMovementPart import MissileMovementPart
from Core.Parts.OnCircleMovementPart import OnCircleMovementPart
from Core.Parts.Part import Part
from Core.Parts.SpritePart import SpritePart
from Core.Parts.TransformPart import TransformPart
from Core.Systems.CollisionSystem import CollisionLayer
from ResourceUtils import load_image, load_sound


class MissileLauncherPart(Part):
    def __init__(self, entity_manager, attack_rate=1.0, damage=10):
        super().__init__()
        self.entity_manager = entity_manager
        self.target = None

        self.attack_rate = attack_rate
        self.cooldown = attack_rate

        self.damage = damage

        self.movement_part = None

    def initialize(self):
        self.transform = self.entity.get(TransformPart)
        self.movement_part = self.entity.get(OnCircleMovementPart)

        self.cooldown = self.attack_rate

    def create_missile(self, velocity):
        missile = Entity('Missile - {}'.format(self.entity.name))
        missile.attach(SpritePart(load_image('missile.png', -1)))

        launch_audio = AudioSourcePart(load_sound('missile_launch.ogg'))
        impact_audio = AudioSourcePart(load_sound('explosion.ogg'))

        missile.attach(TransformPart())
        missile.attach(BoxColliderPart(CollisionLayer.Projectile))
        missile.attach(MissileMovementPart(velocity, target=self.target))
        missile.attach(MissileControlPart(self.entity_manager, self.damage,
                                          {'launch': launch_audio, 'impact': impact_audio}))
        return missile

    def launch_missile(self, velocity):

        if self.cooldown:
            return

        missile = self.create_missile(velocity)
        missile.get(TransformPart).position = self.missile_init_pos
        self.entity_manager.add(missile)

        self.cooldown = self.attack_rate

    def update(self, delta):
        if self.cooldown:
            self.cooldown = max(0, self.cooldown - delta)

    @property
    def missile_init_pos(self):
        up = normalize(sub(self.transform.position, self.movement_part.planet_transform.position))
        return add(self.transform.position, scale(up, 50))
