from Core.MathUtils import radians_to_degrees, GRAD
from Core.Parts.Part import Part
from Core.Parts.TransformPart import TransformPart

import math


class MissileMovementPart(Part):
    def __init__(self, init_vel, target=None):
        super().__init__()
        self.init_vel = init_vel
        self.init_pos = None
        self.alive_time = 0.0
        self.homing_start_time = 0.3
        self.target = target
        self.angle = 0
        self.ddx = 0
        self.ddy = 0
        self.dx = 0
        self.dy = 0
        self.speed = 200

    def initialize(self):
        self.transform = self.entity.get(TransformPart)
        self.init_pos = self.transform.position
        self.alive_time = 0.0
        self.angle = 0
        self.ddx = self.init_vel[0]
        self.ddy = self.init_vel[1]
        self.align_to_movement_init_vel()

    def update(self, delta):
        self.alive_time += delta

        if self.alive_time < self.homing_start_time:
            self.align_to_movement_init_vel()
        else:
            if self.target:
                ix = self.target.position[0] - self.transform.position[0]
                iy = self.target.position[1] - self.transform.position[1]
                self.angle = radians_to_degrees(math.atan2(iy, - ix)) + 90
            self.ddx = -math.sin(self.angle * GRAD)
            self.ddy = -math.cos(self.angle * GRAD)

        self.dx = self.ddx * self.speed
        self.dy = self.ddy * self.speed

        self.transform.move_by((self.dx * delta, self.dy * delta))
        self.transform.rotate_to(self.angle)

    def align_to_movement_init_vel(self):
        self.angle = math.atan2(-self.init_vel[0], -self.init_vel[1]) / math.pi * 180.0
        self.transform.rotate_to(self.angle)
