import math

from Core.MathUtils import sub, normalize
from Core.Parts.Part import Part
from Core.Parts.TransformPart import TransformPart
from math import sin, cos, pi


# TODO: should not take planet as a parameter
class OnCircleMovementPart(Part):
    def __init__(self, planet, radius):
        super().__init__()
        self.planet = planet
        self.planet_transform = None
        self.radius = radius

        self.ang_vel_rad = math.radians(planet.spin_speed * -planet.spin_dir)
        self.angle = 0

        self.movement_modifier = 0

    def initialize(self):
        self.transform = self.entity.get(TransformPart)
        self.planet_transform = self.planet.entity.get(TransformPart)
        self.movement_modifier = 0

        self.rotate(1)

    def update(self, delta):
        planet_to_pos_vec = normalize(sub(self.planet_transform.position, self.transform.position))
        sprite_angle = math.atan2(planet_to_pos_vec[0], planet_to_pos_vec[1]) / math.pi * 180.0
        self.transform.rotate_to(sprite_angle)

    def rotate(self, delta):
        center = self.planet_transform.position
        x_0 = center[0]
        y_0 = center[1]

        added = self.ang_vel_rad * delta

        self.angle += added + self.movement_modifier

        x = self.radius * cos(self.angle) + x_0
        y = self.radius * sin(self.angle) + y_0

        if self.angle >= pi * 2:
            self.angle = 0
        elif self.angle <= 0:
            self.angle = pi * 2

        self.transform.move((x, y))
