from Core.MathUtils import normalize, sub
from Core.Parts.MissileLauncherPart import MissileLauncherPart
from Core.Parts.OnCircleMovementPart import OnCircleMovementPart
from Core.Parts.Part import Part
from Core.Parts.TransformPart import TransformPart
from Core.Parts.VesselPart import VesselPart
from Core.Systems.InputManager import InputType
from pygame.locals import K_a, K_d, KEYDOWN, K_SPACE, KEYUP


class PlayerVesselInputPart(Part):
    def __init__(self, input_manager, key_codes):
        super().__init__()

        self.movement_part = None
        self.missile_launcher = None
        self.vessel = None

        self.key_left = key_codes['left']
        self.key_right = key_codes['right']
        self.key_fire = key_codes['fire']

        self.b1 = 0
        self.b2 = 0

        self.directions = {self.key_left: -1, self.key_right: 1, 0: 0}

        input_manager.register(InputType.BUTTON, self.on_movement_input)

    def initialize(self):
        self.movement_part = self.entity.get(OnCircleMovementPart)
        self.transform = self.entity.get(TransformPart)
        self.missile_launcher = self.entity.get(MissileLauncherPart)
        self.vessel = self.entity.get(VesselPart)

    def on_movement_input(self, *args):
        if len(args) == 2:
            if args[0] == self.key_left or args[0] == self.key_right:
                self.resolve_horizontal_axis(args[0], args[1])
            elif args[0] == self.key_fire and args[1] == KEYDOWN:
                if self.missile_launcher:
                    self.missile_launcher.launch_missile(self.vec_up)
            pressed = self.b2 if self.b2 else self.b1
            pressed = 0 if not pressed else pressed
            self.vessel.mov_dir = self.directions[pressed]

    def resolve_horizontal_axis(self, key, t):
        if not self.b1:
            if t == KEYDOWN:
                self.b1 = key
        else:
            if self.b1 == key:
                if t == KEYUP:
                    self.b1 = self.b2
                    self.b2 = 0
            else:
                self.b2 = 0 if t == KEYUP else key


    @property
    def vec_up(self):
        return normalize(sub(self.transform.position, self.movement_part.planet_transform.position))
