import pygame

from Core.Parts.Part import Part
from Core.Parts.TransformPart import TransformPart


class SpritePart(Part):
    def __init__(self, sprite, flipped_sprite=None):
        super().__init__()
        self.sprite = sprite
        self.dirty_sprite = DirtySprite(sprite)
        self.curr_angle = 0
        self.flipped_sprite = flipped_sprite
        self._is_flipped = False
        self.flip_req = False

    def initialize(self):
        self.transform = self.entity.get(TransformPart)
        self.curr_angle = 0

    def update(self, delta):
        if self.flip_req:
            if self.flipped_sprite:
                if self._is_flipped:
                    self.dirty_sprite.set_sprite(self.sprite, False)
                else:
                    self.dirty_sprite.set_sprite(self.flipped_sprite, False)
                self._is_flipped = not self._is_flipped
                self.dirty_sprite.dirty = 1
                self.dirty_sprite.update()
            self.flip_req = False

        if self.dirty_sprite.rect.center != self.transform.position:
            self.dirty_sprite.rect.center = self.transform.position
            self.dirty_sprite.dirty = 1
        if self.curr_angle != self.transform.angle_deg:
            self.curr_angle = self.transform.angle_deg
            self.dirty_sprite.rotate(self.curr_angle)
            self.dirty_sprite.dirty = 1

    def move_ip(self, pos):
        self.dirty_sprite.rect.move_ip(pos)
        self.dirty_sprite.dirty = 1

    def flip(self):
        self.flip_req = True

    @property
    def image(self):
        return self.dirty_sprite.image

    @property
    def rect(self):
        return self.dirty_sprite.rect

    @property
    def is_flipped(self):
        return self._is_flipped


class DirtySprite(pygame.sprite.DirtySprite):
    def __init__(self, sprite):
        pygame.sprite.DirtySprite.__init__(self)
        self.image, self.rect = None, None
        self.image0 = None
        self.set_sprite(sprite)

    def set_sprite(self, sprite, init_rect=True):
        self.image, rect = sprite
        if init_rect:
            self.rect = rect
        self.image0 = self.image

    def rotate(self, angle_deg):
        old_rect = self.rect.center
        self.image = pygame.transform.rotate(self.image0, angle_deg)
        self.rect = self.image.get_rect()
        self.rect.center = old_rect
