import pygame

import ResourceUtils
from Core.Parts.Part import Part


ScreenSizeX = 800
ScreenSizeY = 600


class TextPart(Part):
    def __init__(self, msg, col=(225, 225, 225), pos=(0, 0), alignment=('left', 'top')):
        super().__init__()
        self.color = col
        self.dirty_sprite = None
        self.screen_size = None
        self.alignment = alignment
        self.position = pos
        self.text = msg

    def initialize(self):
        self.dirty_sprite = Text(self.text, self.color, self.position, self.alignment)

    def set_text(self, msg):
        self.dirty_sprite.set_text(msg, self.color)

    def top_left(self, top_left):
        self.dirty_sprite.rect.topleft = top_left

    def top_right(self, top_right):
        self.dirty_sprite.rect.topright = top_right


class Text(pygame.sprite.DirtySprite):
    def __init__(self, msg, color, pos, alignment):
        self.alignment = alignment
        self.pos = pos
        self._layer = 1
        pygame.sprite.DirtySprite.__init__(self)

        self.image = None
        self.rect = None

        self.set_text(msg, color)

    def set_text(self, msg, color=(0, 0, 0)):
        self.image = self.write(msg, color)
        self.rect = self.image.get_rect()

        align_x = self.alignment[0]
        align_y = self.alignment[1]

        if align_x == 'left':
            self.rect.left = self.pos[0]
        elif align_x == 'right':
            self.rect.right = self.pos[0]
        elif align_x == 'center':
            self.rect.left = (ScreenSizeX - self.image.get_width()) / 2

        if align_y == 'top':
            self.rect.top = self.pos[1]
        elif align_y == 'bottom':
            self.rect.right = self.pos[1]
        elif align_y == 'center':
            self.rect.top = (ScreenSizeY - self.image.get_height()) / 2

        # HACK
        if align_y:
            self.rect.y += self.pos[1]

    @staticmethod
    def write(msg, color):
        font = ResourceUtils.load_font('fsb.ttf')
        text = font.render(msg, True, color)
        text = text.convert_alpha()
        return text
