from Core.MathUtils import sign
from Core.Parts.BoxColliderPart import BoxColliderPart
from Core.Parts.OnCircleMovementPart import OnCircleMovementPart
from Core.Parts.Part import Part
from Core.Parts.SpritePart import SpritePart


class VesselPart(Part):
    def __init__(self, speed, sound_components):
        super().__init__()

        self.movement_part = None
        self.speed = speed
        self.mov_dir = 0
        self.engine_audio = sound_components['engine_audio']
        self.flipped_dir = -1

    def initialize(self):
        self.collider = self.entity.get(BoxColliderPart)
        self.movement_part = self.entity.get(OnCircleMovementPart)
        self.sprite = self.entity.get(SpritePart)
        self.collider.rect_inflate = (-15, -10)
        self.mov_dir = 0

    def update(self, delta):
        if self.mov_dir:
            self.engine_audio.start_playback(True)
        else:
            self.engine_audio.stop_playback()

        self.movement_part.movement_modifier = self.mov_dir * self.speed * delta
        self.movement_part.rotate(delta)

        mov_sign = sign(self.mov_dir)

        if mov_sign != 0:
            should_be_flipped = (mov_sign == self.flipped_dir)
            if should_be_flipped != self.sprite.is_flipped:
                self.sprite.flip()

    def on_collision_with(self, other_collider):
        pass



