from enum import Enum

from Core.Parts.BoxColliderPart import BoxColliderPart
from Core.Parts.CircleColliderPart import CircleColliderPart


class CollisionLayer(Enum):
    Projectile = 1
    Vessel = 2
    Planet = 3


class CollisionSystem:
    def __init__(self, entity_manager):
        self.entities = entity_manager.entities
        self.layer_collisions = {
            CollisionLayer.Projectile: [CollisionLayer.Projectile, CollisionLayer.Planet],
        }
        self.active_collision_layers = self.layer_collisions.keys()

    def update(self):
        colliders = []
        for e in self.entities:
            if e.has(BoxColliderPart):
                colliders.append(e.get(BoxColliderPart))
            elif e.has(CircleColliderPart):
                colliders.append(e.get(CircleColliderPart))

        for col in colliders:
            if col.layer in self.active_collision_layers:
                for other in colliders:
                    if col != other:
                        if col.collides_with_collider(other):
                            col.entity.on_collision_with(other)
                            other.entity.on_collision_with(col)


