import pygame
from pygame.locals import *
from enum import Enum

from Core.EventManager import EventManager


class InputType(Enum):
    BUTTON = 1,
    MOUSE_MOTION = 2


class InputManager(EventManager):
    def __init__(self):
        super().__init__()
        self.event_types_handled = [MOUSEBUTTONUP, MOUSEBUTTONDOWN, QUIT]
        self.event_keys_handled = [K_ESCAPE, K_a, K_d, K_SPACE, K_RETURN, K_1, K_2, K_LEFT, K_RIGHT, K_RCTRL]

    def update(self):
        # Handle Input Events
        for event in pygame.event.get():
            if event.type == MOUSEMOTION:
                self.invoke(InputType.MOUSE_MOTION, pygame.mouse.get_pos())
            elif hasattr(event, 'key') and event.key in self.event_keys_handled:
                self.invoke(InputType.BUTTON, event.key, event.type)
            elif event.type in self.event_types_handled:
                self.invoke(InputType.BUTTON, event.type)
